/* upsmon - monitor power status over the 'net (talks to upsd via UDP)

   Copyright (C) 1998  Russell Kroll <rkroll@exploits.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              
 */

#include <unistd.h>
#include <stdio.h>
#include <syslog.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "upsfetch.h"

/* bit values that get OR'ed later */
#define UPS_CALIBRATION	1
#define UPS_SLEEPING	2
#define UPS_UNKNOWN	4
#define UPS_ONLINE	8
#define UPS_ONBATT	16
#define UPS_UNKNOWN2	32
#define UPS_BATTLOW	64
#define UPS_CHARGING	128

#define ALERTTIME	30		/* after this many seconds on battery,
					   do something about it */

extern void wall();
void readconf(char *configfile, char *passwd, char **upsport, char **shutdown);

	char	passwd[16], *upsport, *shutdowncmd;

void notify(char *notice)
{
	wall (notice, 1);
}

void nowonbattery()
{
	syslog (LOG_NOTICE, "UPS is on battery");	
	notify ("UPS is on battery\n");
}

void notonbattery()
{
	syslog (LOG_NOTICE, "UPS is no longer on battery");
	notify ("UPS is no longer on battery\n");
}

void flee()
{
	syslog (LOG_CRIT, "Initiating automatic power-fail shutdown");
	notify ("Initiating automatic power-fail shutdown\n");

	if (getuid() != 0) {
		syslog (LOG_ALERT, "Not root, unable to shutdown system");
		exit (1);
	}
		
	notify ("Auto logout and shutdown in 15 seconds!\n");
	sleep (15);
	
	system (shutdowncmd);
	exit (1);
}

void become_daemon()
{
	int	pid;
	
	close (0);
	close (1);
	close (2);

	if ((pid = fork()) < 0) {
		perror ("Unable to fork");
		exit (1);
	}
	if (pid != 0) exit (0);
}

void onbattlongtime()
{
	char	nottmp[80];

	syslog (LOG_ALERT, "UPS has been on battery for more than %i seconds",
		ALERTTIME);

	sprintf (nottmp, "Prolonged power outage (longer than %i seconds)\n", 
		 ALERTTIME);

	notify (nottmp);
}

int main(int argc, char *argv[])  
{
	char	monhost[256];
	int	status;
	int	on_battery = 0, noresp = -1;
	int	onbatttime = 0, longtimewarn = 0;
	struct	timeval		tv;
	struct	timezone	tz;
	char	configfile[128];
	upsinfo ups;
	
	printf ("upsmon by Russell Kroll <rkroll@exploits.org>\n");

	if (argc != 2) {
		printf ("Usage: %s <host to monitor>     Example: %s 10.2.254.1\n", argv[0], argv[0]);
		exit (1);
	}

	sprintf (configfile, "%s/ups.conf", BASEPATH);
	readconf (configfile, passwd, &upsport, &shutdowncmd);

	if ((shutdowncmd == NULL) || (strlen(shutdowncmd) == 0)) {
		printf ("SHUTDOWN must be configured in %s\n", configfile);
		exit (1);
	}

	if (strlen(passwd) == 0) {
		printf ("PASSWD must be configured in %s\n", configfile);
		exit (1);
	}

	/* upsport isn't used by upsmon .. */

	strncpy (monhost, argv[1], sizeof(monhost) - 1);

	printf ("Monitoring UPS on %s\n", monhost);
	become_daemon(); 
	
	openlog ("upsmon", LOG_PID, LOG_INFO);
	syslog (LOG_INFO, "Monitoring UPS on %s", monhost);

	while (1) {
		if (fetch (monhost, &ups) > 0) {
			if (noresp == -1) {	/* ups is there, first time */
				noresp = 0;
				syslog (LOG_INFO, "Communication with UPS established");
			}

			if (noresp == 1) {	/* ups is back */
				noresp = 0;
				syslog (LOG_INFO, "Communication with UPS re-established");
			}

			status = strtol (ups.status, 0, 16);

			if (status & UPS_ONBATT) {	/* on battery */
				gettimeofday(&tv, &tz);
		
				if ((onbatttime != 0) && (longtimewarn == 0) &&
				   ((tv.tv_sec - onbatttime) > ALERTTIME)) {
					onbattlongtime();
					longtimewarn = 1;
				}					

				if (!on_battery) {
					on_battery = 1;
					nowonbattery();
					onbatttime = tv.tv_sec;
				}
			}	/* not on battery currently */
			else
				if (on_battery) {
					on_battery = 0;
					notonbattery();
					onbatttime = 0;
					longtimewarn = 0;
				}

			if ((status & UPS_BATTLOW) && (status & UPS_ONBATT))
				flee();

		}
		else {	/* fetch failed */
			if (noresp == 0) {  /* ups isn't there, first time */
				syslog (LOG_INFO, "Communication with UPS lost");
				noresp = 1;
			}
		}
		sleep (5);
	}

	return (1);
}
